/**
 * Funkcje zwizane z obsug przycisku.
 */
#ifdef DEBUG
#	define isBUTTON(widget) (assert(widget->type == BUTTON));
#else
#	define isBUTTON(widget)
#endif

#include "Button.colors"

/** @brief Przycisk.
 *
 * Po naciniciu przycisku wywoywana jest zarejestrowana funkcja #callback.
 */
typedef struct uiButton {
	button_callback callback;	/**< funkcja wywoywana przy naciniciu przycisku */
	char* label;			/**< nazwa przycisku */
	GLenum halign;			/**< wyrwnanie napisu w poziomie */
	GLenum valign;			/**< wyrwnanie napisu w pionie */
} uiButton;

static void button_draw(pWidget widget, GLint dx, GLint dy) {
	uiButton* button;
	
	if (widget->visible == GL_FALSE) return;

	button = (uiButton*)widget->extra;
	if (widget->active)
		draw_button_face(widget->x + dx, widget->y + dy, widget->w, widget->h,
				 button->label, button->halign, button->valign,
				 button_color_active, button_text_active);
	else
		draw_button_face(widget->x + dx, widget->y + dy, widget->w, widget->h,
				 button->label, button->halign, button->valign,
				 button_color_inactive, button_text_inactive);
}

static void button_activate(pWidget widget, GLint x, GLint y) {
	uiButton* button;

	isBUTTON(widget);

	button = (uiButton*)widget->extra;
	if (widget->active && button->callback)
		button->callback(widget);
}

/**
 * Utworzenie nowego przycisku.
 *
 * @param widget	struktura danych zwrcona przez uiNewWidget
 * @param callback	wskanik do funkcji typu button_callback
 * @param label		tekst wywietlany na przycisku
 * @param halign	wyrwnanie napisu w poziomie: ALIGN_CENTER (domylnie),
 * 			ALIGN_LEFT, ALIGN_RIGHT
 * @param valign	wyrwnanie napisu w pionie: ALIGN_CENTER (domylnie),
 * 			ALIGN_LEFT, ALIGN_RIGHT
 *
 * @return pWidget	widget
 */
pWidget uiNewButton(pWidget widget, button_callback callback, const char* label, GLenum halign, GLenum valign) {
	uiButton* new;
	
	new = (uiButton*)malloc(sizeof(uiButton));
	new->callback	= callback;
	new->label	= mystrdup(label);
	new->halign	= SetOption3(halign, ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT);
	new->valign	= SetOption3(valign, ALIGN_CENTER, ALIGN_TOP, ALIGN_BOTTOM);
	
	widget->mbp	= &button_activate;
	widget->draw	= &button_draw;
	widget->extra	= (void*)new;
	widget->type	= BUTTON;

	return widget;
}
