#ifdef DEBUG
#	define isFSLIDER(widget) assert(widget->type == FSLIDER);
#else
#	define isFSLIDER(widget)
#endif

#include "Slider.colors"

typedef struct uiFSlider {
	fslider_callback callback;
	GLfloat		x;
	GLfloat		min;
	GLfloat		max;
	GLenum		orientation;
	GLfloat		color[4];
} uiFSlider;

static void fslider_activate(pWidget widget, GLint x, GLint y) {
	uiFSlider* slider;
	GLfloat t;

	isFSLIDER(widget);
	
	slider = (uiFSlider*)widget->extra;
	
	if (widget->active) {
		if (slider->orientation == SLIDER_HORIZONTAL)
			t = (float)(x - widget->x) / (widget->w);
		else
			t = (float)(y - widget->y) / (widget->h);

		slider->x = clampf( lerp(slider->min, slider->max, t), slider->min, slider->max);
		if (slider->callback)
			slider->callback(widget, slider->x);
	}
}

static void fslider_draw(pWidget widget, GLint dx, GLint dy) {
	uiFSlider* slider;
	GLfloat    t;
	GLuint     width;
	GLuint     height;
	static char buffer[13];

	isFSLIDER(widget);
	
	if (widget->visible == GL_FALSE) return;

	slider = (uiFSlider*)widget->extra;

	t = (slider->x - slider->min) / (slider->max - slider->min);
	if (slider->orientation == SLIDER_HORIZONTAL) {
		sprintf(buffer, "%0.1f", slider->x);
		width = (int)(t * widget->w);

		if (widget->active) {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_active);
			draw_rectangle(widget->x + dx, widget->y + dy, width, widget->h, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_active);
		}
		else {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_inactive);
			draw_rectangle(widget->x + dx, widget->y + dy, width, widget->h, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_inactive);
		}
	}
	else {
		sprintf(buffer, "%0.1f", slider->x);
		height = (int)(t * widget->h);

		if (widget->active) {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_active);
			draw_rectangle(widget->x + dx, widget->y + dy, widget->w, height, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_active);
		}
		else {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_inactive);
			draw_rectangle(widget->x + dx, widget->y + dy, widget->w, height, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_inactive);
		}
	}
}

pWidget uiNewFSlider(pWidget widget, fslider_callback callback, GLfloat x, GLfloat min, GLfloat max, GLenum orientation, GLfloat r, GLfloat g, GLfloat b) {
	uiFSlider* new;
	
	new = (uiFSlider*)malloc(sizeof(uiFSlider));
	assert(new != NULL);
	
	new->callback	= callback;
	new->x		= clampf(x, min, max);
	new->min	= min;
	new->max	= max;
	new->orientation	= SetOption2(orientation, SLIDER_HORIZONTAL, SLIDER_VERTICAL);
	
	new->color[0]	= r;
	new->color[1]	= g;
	new->color[2]	= b;
	new->color[3]	= 1.0;
			
	widget->mbp	= &fslider_activate;
	widget->draw	= &fslider_draw;
	widget->extra	= (void*)new;
	widget->type	= FSLIDER;
	return widget;
}

GLint uiFSliderGetValue(pWidget widget) {
	isFSLIDER(widget);
	return ((uiFSlider*)widget->extra)->x;
}

void uiFSliderSetValue(pWidget widget, GLfloat x) {
	isFSLIDER(widget);
#define slider ((uiFSlider*)widget->extra)
	slider->x = clampf(x, slider->min, slider->max);
#undef slider
}

GLfloat uiFSliderGetMin(pWidget widget) {
	isFSLIDER(widget);
	return ((uiFSlider*)widget->extra)->min;
}

GLfloat uiFSliderGetMax(pWidget widget) {
	isFSLIDER(widget);
	return ((uiFSlider*)widget->extra)->max;
}

#undef isFSLIDER
