#ifdef DEBUG
#	define isISLIDER(widget) assert(widget->type == ISLIDER);
#else
#	define isISLIDER(widget)
#endif

#include "Slider.colors"

typedef struct uiISlider {
	islider_callback callback;
	GLint		x;
	GLint		min;
	GLint		max;
	GLenum		orientation;
	GLfloat		color[4];
} uiISlider;

static void islider_activate(pWidget widget, GLint x, GLint y) {
	uiISlider* slider;
	GLfloat t;

	isISLIDER(widget);
	
	slider = (uiISlider*)widget->extra;
	
	if (widget->active) {
		if (slider->orientation == SLIDER_HORIZONTAL)
			t = (float)(x - widget->x) / (widget->w);
		else
			t = (float)(y - widget->y) / (widget->h);

		slider->x = clampi( (int)lerp(slider->min, slider->max, t), slider->min, slider->max);
		if (slider->callback)
			slider->callback(widget, slider->x);
	}
}

static void islider_draw(pWidget widget, GLint dx, GLint dy) {
	uiISlider* slider;
	GLfloat    t;
	GLuint     width;
	GLuint     height;
	static char buffer[9];

	isISLIDER(widget);
	
	if (widget->visible == GL_FALSE) return;

	slider = (uiISlider*)widget->extra;

	t = (GLfloat)(slider->x - slider->min) / (slider->max - slider->min);
	if (slider->orientation == SLIDER_HORIZONTAL) {
		sprintf(buffer, "%d", slider->x);
		width = (int)(t * widget->w);
		
		if (widget->active) {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_active);
			draw_rectangle(widget->x + dx, widget->y + dy, width, widget->h, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_active);
		}
		else {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_inactive);
			draw_rectangle(widget->x + dx, widget->y + dy, width, widget->h, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_inactive);
		}
	}
	else {
		sprintf(buffer, "%d", slider->x);
		height = (int)(t * widget->h);

		if (widget->active) {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_active);
			draw_rectangle(widget->x + dx, widget->y + dy, widget->w, height, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_active);
		}
		else {
			draw_shaded_rectangle(widget->x + dx, widget->y + dy, widget->w, widget->h, slider_back_inactive);
			draw_rectangle(widget->x + dx, widget->y + dy, widget->w, height, slider->color);
			draw_aligned_string(widget->x + dx, widget->y + dy, widget->w, widget->h, buffer, ALIGN_CENTER, ALIGN_CENTER, slider_text_inactive);
		}
	}
}

pWidget uiNewISlider(pWidget widget, islider_callback callback, GLint x, GLint min, GLint max, GLenum orientation, GLfloat r, GLfloat g, GLfloat b) {
	uiISlider* new;
	
	new = (uiISlider*)malloc(sizeof(uiISlider));
	assert(new != NULL);
	
	new->callback	= callback;
	new->x		= clampi(x, min, max);
	new->min	= min;
	new->max	= max;
	new->orientation	= SetOption2(orientation, SLIDER_HORIZONTAL, SLIDER_VERTICAL);
	
	new->color[0]	= r;
	new->color[1]	= g;
	new->color[2]	= b;
	new->color[3]	= 1.0;
			
	widget->mbp	= &islider_activate;
	widget->draw	= &islider_draw;
	widget->extra	= (void*)new;
	widget->type	= ISLIDER;
	return widget;
}

int xxxxxx(float a) {
	return a > 0.0;
}

GLint uiISliderGetValue(pWidget widget) {
	isISLIDER(widget);
	return ((uiISlider*)widget->extra)->x;
}

void uiISliderSetValue(pWidget widget, GLint x) {
	isISLIDER(widget);
	((uiISlider*)widget->extra)->x = x;
}

GLint uiISliderGetMin(pWidget widget) {
	isISLIDER(widget);
	return ((uiISlider*)widget->extra)->min;
}

GLint uiISliderGetMax(pWidget widget) {
	isISLIDER(widget);
	return ((uiISlider*)widget->extra)->max;
}

#undef isISLIDER
