#ifdef DEBUG
#	define isLABEL(widget) (assert(widget->type == LABEL));
#else
#	define isLABEL(widget)
#endif

typedef struct uiLabel {
	char**	lines;
	GLuint	n;

	GLenum	halign;
	GLuint	interlinesep;
	GLfloat	textcolor[4];
} uiLabel;

static void label_draw(pWidget widget, GLint dx, GLint dy) {
	uiLabel	*label;
	GLuint	i, y;

	label = (uiLabel*)widget->extra;

	y = widget->y + dy;
	for (i=0; i < label->n; i++) {
		draw_aligned_string(widget->x + dx, y, widget->w, widget->h, label->lines[i], label->halign, ALIGN_BOTTOM, label->textcolor);
		y += label->interlinesep;
	}
}

pWidget uiNewLabel(pWidget widget, GLenum halign, GLenum interlinesep, GLfloat r, GLfloat g, GLfloat b, ...) {
	uiLabel*	new;
	GLuint		n, i;
	va_list		ap;

	new = (uiLabel*)malloc(sizeof(uiLabel));
	assert(new != NULL);
	
	new->halign		= SetOption3(halign, ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT);
	new->interlinesep	= interlinesep;
	new->textcolor[0]	= clampf(r, 0.0, 1.0);
	new->textcolor[1]	= clampf(g, 0.0, 1.0);
	new->textcolor[2]	= clampf(b, 0.0, 1.0);
	new->textcolor[3]	= 1.0;

	/* count optional arguments */
	va_start(ap, b);
	n = 0;
	while (va_arg(ap, pWidget) != NULL)
		n++;
	va_end(ap);

	/* allcoate space for children */
	new->n		= n;
	new->lines	= (char**)malloc(sizeof(char*) * n);
	assert(new->lines != NULL);

	va_start(ap, b);
	for (i=0; i<n; i++)
		new->lines[n-i-1] = mystrdup(va_arg(ap, char*));
	va_end(ap);

	widget->draw		= &label_draw;
	widget->extra		= (void*)new;
	widget->type		= LABEL;

	return widget;
}	
