#ifdef DEBUG
#	define isRADIOBUTTONS(widget) (assert(widget->type == RADIOBUTTONS));
#else
#	define isRADIOBUTTONS(widget)
#endif

#include "RadioButtons.colors"

typedef struct uiRadioButtons {
	radiobuttons_callback callback;
	GLuint option;
	GLuint n;
	char** labels;
	GLenum halign;
	GLenum valign;
	GLenum orientation;
} uiRadioButtons;

static void radiobuttons_activate(pWidget widget, GLint x, GLint y) {
	uiRadioButtons* radio;
	GLuint dx, dy;

	radio = (uiRadioButtons*)widget->extra;
	if (widget->active == GL_FALSE)
		return;

	if (radio->orientation == RADIOBUTTONS_VERTICAL) {
		dy = widget->h / radio->n;
		radio->option = radio->n - 1 - clampi( (y - widget->y)/dy, 0, radio->n);
	}
	else {
		dx = widget->w / radio->n;
		radio->option = clampi( (x - widget->x)/dx, 0, radio->n);
	}

	if (radio->callback)
		radio->callback(widget, radio->option);
}

static void radiobuttons_draw(pWidget widget, GLint deltax, GLint deltay) {
	uiRadioButtons* radio;
	GLuint i;
	GLuint x,y,dx,dy;
	GLfloat *back_color_on, *back_color_off, *text_color;

	if (widget->visible == GL_FALSE) return;

	if (widget->active) {
		back_color_on	= radiobuttons_back_active_on;
		back_color_off	= radiobuttons_back_active_off;
		text_color	= radiobuttons_text_active;
	}
	else {
		back_color_on	= radiobuttons_back_inactive_on;
		back_color_off	= radiobuttons_back_inactive_off;
		text_color	= radiobuttons_text_inactive;
	}

	radio   = (uiRadioButtons*)widget->extra;
	if (radio->orientation == RADIOBUTTONS_VERTICAL) {
		dy = widget->h / radio->n;
		y  = widget->y + deltay;
		for (i=0; i < radio->n; i++) {
			if (radio->n - i - 1 == radio->option) {
				draw_button_face(widget->x + deltax, y, widget->w, dy,
						radio->labels[i], radio->halign, radio->valign,
						back_color_on, text_color);

			}
			else {
				draw_button_face(widget->x + deltax, y, widget->w, dy,
						radio->labels[i], radio->halign, radio->valign,
						back_color_off, text_color);
			}
			y += dy;
		}
	}
	else {
		dx = widget->w / radio->n;
		x  = widget->x + deltax;
		for (i=0; i < radio->n; i++) {
			if (i == radio->option) {
				draw_button_face(x, widget->y + deltay, dx, widget->h,
						radio->labels[i], radio->halign, radio->valign,
						back_color_on, text_color);

			}
			else {
				draw_button_face(x, widget->y + deltay, dx, widget->h,
						radio->labels[i], radio->halign, radio->valign,
						back_color_off, text_color);
			}
			x += dx;
		}
	}
}

pWidget uiNewRadioButtons(pWidget widget, radiobuttons_callback callback, GLenum halign, GLenum valign, GLenum orientation, ...) {
	uiRadioButtons* new;
	va_list ap;
	GLuint	i, n=0;
	
	new			= (uiRadioButtons*)malloc(sizeof(uiRadioButtons));
	new->option		= 0;
	new->callback		= callback;
	new->halign		= SetOption3(halign, ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT);
	new->valign		= SetOption3(valign, ALIGN_CENTER, ALIGN_TOP, ALIGN_BOTTOM);
	new->orientation	= SetOption2(orientation, RADIOBUTTONS_VERTICAL, RADIOBUTTONS_HORIZONTAL);

	va_start(ap, orientation);
	n = 0;
	while (va_arg(ap, char*) != NULL)
		n++;
	va_end(ap);
	new->n			= n;
	new->labels		= (char**)malloc(sizeof(char*) * n);

	va_start(ap, orientation);
		if (new->orientation == RADIOBUTTONS_HORIZONTAL)
			for (i=0; i<n; i++)
				new->labels[i] = mystrdup(va_arg(ap, char*));
		else
			for (i=0; i<n; i++)
				new->labels[n-i-1] = mystrdup(va_arg(ap, char*));

	va_end(ap);

	widget->mbp		= &radiobuttons_activate;
	widget->draw		= &radiobuttons_draw;
	widget->extra		= (void*)new;
	widget->type		= RADIOBUTTONS;
	
	return widget;
}

GLuint uiRadioButtonsGetOption(pWidget widget) {
	return ((uiRadioButtons*)widget->extra)->option;
}

void uiRadioButtonsSetOption(pWidget widget, GLuint option) {
	uiRadioButtons* radio;
	radio = (uiRadioButtons*)widget->extra;
	
	if (option < radio->n)
		radio->option = option;
}
