/** @file
 *
 * @brief Gwny plik vso-gui
 *
 * @mainpage
 * 
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <assert.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glut.h>

#include "gui.h"

pWidget uiNewWidget(GLuint id, GLint x, GLint y, GLuint w, GLuint h) {
	pWidget new;
	new = (pWidget)malloc(sizeof(uiWidget));
	
	new->type	= NONE;
	new->visible	= GL_TRUE;
	new->active	= GL_TRUE;

	new->id		= id;
	new->x		= x;
	new->y		= y;
	new->w		= w;
	new->h		= h;
	
	new->extra	= NULL;
	new->mbp	= NULL;
	new->draw	= NULL;
	
	return new;
}

static pWidget* gui_widget_list;

void uiHide(pWidget w)
	{w->visible = GL_FALSE;}
	
void uiShow(pWidget w)
	{w->visible = GL_TRUE;}

void uiActive(pWidget w)
	{w->active = GL_TRUE;}
	
void uiDeactive(pWidget w)
	{w->active = GL_FALSE;}

static GLuint		window_width;
static GLuint		window_height;

#include "utils.c"

#include "Label.c"
#include "LabelRef.c"
#include "Button.c"
#include "CheckBox.c"
#include "RadioButtons.c"
#include "ISlider.c"
#include "FSlider.c"
#include "Container.c"

void uiCenterWidget(pWidget widget) {
	widget->x = ((GLint)window_width - (GLint)widget->w)/2;
	widget->y = ((GLint)window_height - (GLint)widget->h)/2;
}

/* zaoenie, e wszystkie widgety zostay stworzone przez uiNewXXX */
static void destroy_widget(pWidget widget) {
	uiButton	*button;
	uiCheckBox	*checkbox;
	uiRadioButtons	*radiobuttons;
	uiContainer	*container;
	uiLabel		*label;

	GLuint		i;
	
	switch (widget->type) {
		case BUTTON:
			button = (uiButton*)widget->extra;
			free(button->label);
			break;
		case CHECKBOX:
			checkbox = (uiCheckBox*)widget->extra;
			free(checkbox->label);
			break;
		case RADIOBUTTONS:
			radiobuttons = (uiRadioButtons*)widget->extra;
			for (i=0; i < radiobuttons->n; i++)
				free(radiobuttons->labels[i]);
			free(radiobuttons->labels);
			break;
		case ISLIDER:
			break;
		case FSLIDER:
			break;
		case CONTAINER:
			container = (uiContainer*)widget->extra;
			for (i=0; i < container->n; i++)
				destroy_widget(container->children[i]);
			free(container->children);
			break;
		case LABEL:
			label = (uiLabel*)widget->extra;
			for (i=0; i < label->n; i++)
				free(label->lines[i]);
			free(label->lines);
			break;
		default:
			assert(0);
	}
	free(widget->extra);
	free(widget);
}

static GLuint		gui_widget_count;
static GLint		gui_x;
static GLint		gui_y;
static GLboolean	gui_visible;
static GLuint		window_size;

void ReshapeGUI(GLuint width, GLuint height) {
	window_width	= width;
	window_height	= height;
	window_size	= window_width > window_height ? window_width : window_height;
}

void InitGUI(GLuint width, GLuint height, ...) {
	va_list		ap;
	GLuint 		n, i;
	
	gui_visible		= GL_FALSE;
	gui_x			= 0;
	gui_y			= 0;

	va_start(ap, height);
	n = 0;
	while (va_arg(ap, pWidget) != NULL)
		n++;
	va_end(ap);
	
	gui_widget_count	= n;
	gui_widget_list		= (pWidget*)malloc(sizeof(pWidget) * n);

	va_start(ap, height);
	for (i=0; i<n; i++)
		gui_widget_list[i] = va_arg(ap, pWidget);
	va_end(ap);
	
	ReshapeGUI(width, height);
}

void HideGUI() {gui_visible = GL_FALSE;}
void ShowGUI() {gui_visible = GL_TRUE;}

void DrawGUI() {
	GLuint i;

	if (gui_visible == GL_FALSE) return;

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE_1D);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluOrtho2D(0, window_size, 0, window_size);
	glViewport(0, 0, window_size, window_size);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.375, 0.375, 0.0);

	for (i=0; i<gui_widget_count; i++) {
		gui_widget_list[i]->draw(gui_widget_list[i], gui_x, gui_y);
	}
	glDisable(GL_BLEND);
}

static GLboolean click_mouse(pWidget widget, GLint x, GLint y) {
	uiContainer 	*container;
	GLuint		i;

	if (widget->visible == GL_FALSE)
		return GL_FALSE;

	if (widget->type == CONTAINER) {
		container = (uiContainer*)widget->extra;
		for (i=0; i < container->n; i++)
			if (click_mouse(container->children[i], x - widget->x, y - widget->y))
				return GL_TRUE;
	}
	else
		if (widget->mbp)
			if  ((widget->x <= x && x <= widget->x + widget->w) &&
			     (widget->y <= y && y <= widget->y + widget->h)) {
				widget->mbp(widget, x, y);
				return GL_TRUE;
			}

	return GL_FALSE;
}

void ClickGUI(GLint x, GLint y) {
	GLuint	i;

	if (gui_visible == GL_FALSE)
		return;

	y = window_height - y;
	for (i=0; i<gui_widget_count; i++) {
		if (click_mouse(gui_widget_list[i], x, y))
			return;
	}
}

void DestroyGUI() {
	GLuint i;

	for (i=0; i < gui_widget_count; i++)
		destroy_widget(gui_widget_list[i]);
	free(gui_widget_list);
	gui_widget_count	= 0;
}

/*
void glutClick(int button, int state, int x, int y) {
	if (state == GLUT_DOWN && button == GLUT_LEFT)
		ClickGUI(x,y);
}
*/
