#define DECLARE_RGBA(name, r,g,b,a)	static GLfloat name[4] = {r, g, b, a};
#define DECLARE_RGB(name, r,g,b)	static GLfloat name[4] = {r, g, b, 1.0};
#define DECLARE_GRAYA(name, v,a)	static GLfloat name[4] = {v, v, v, a};
#define DECLARE_GRAY(name, v)		static GLfloat name[4] = {v, v, v, 1.0};

#define SetOption2(value, default, option2) (value == default || value == option2) ? (value) : (default)
#define SetOption3(value, default, option2, option3) ((value) == (default) || (value) == (option2) || (value) == (option3)) ? (value) : (default)

static char* mystrdup(const char* s) {
	char* 	c;
	size_t	n;
	
	n = strlen(s);
	c = (char*)malloc(n+1);
	strncpy(c, s, n+1);
	
	return c;
}

static GLint clampi(GLint x, GLint min, GLint max) {
	if (x < min)
		return min;
	else
		if (x > max)
			return max;
	else
		return x;
}

static GLfloat lerp(GLfloat a, GLfloat b, GLfloat t) {
	return a + t*(b-a);
}

static GLfloat clampf(GLfloat x, GLfloat min, GLfloat max) {
	if (x < min)
		return min;
	else
	if (x > max)
		return max;
	else
		return x;
}

static GLuint string_width(const char* string) {
	GLuint width = 0;
	char* c;

	c = (char*)string;
	while (*c != '\0')
		width += glutBitmapWidth(GLUT_BITMAP_HELVETICA_18, *c++);
	return width;
}

/*
static GLuint string_height(const char* string) {
	return 18;
}
*/
#define string_height(_) 18

static void draw_string(GLint x, GLint y, const char* string, GLfloat *color) {
	char *c;

	c = (char*)string;

	glColor4fv(color);
	glRasterPos2i(x, y);
	while (*c != '\0') {
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *c);
		x += (GLint)glutBitmapWidth(GLUT_BITMAP_HELVETICA_18, *c);
		c++;
	}
}

static void draw_aligned_string(GLint x, GLint y, GLint w, GLint h, const char* string, GLenum halign, GLenum valign, GLfloat* color) {
	GLint dx=0, dy=0;
	switch (halign) {
		case ALIGN_LEFT:
			break;
		case ALIGN_RIGHT:
			dx = w - (int)string_width(string);
			break;
		case ALIGN_CENTER:
			dx = (w - (int)string_width(string))/2;
			break;
	}

	switch (valign) {
		case ALIGN_TOP:
			dy = h - (int)string_height(string);
			break;
		case ALIGN_BOTTOM:
			break;
		case ALIGN_CENTER:
			dy = (h - (int)string_height(string))/2;
			break;
	}

	draw_string(x + dx,y + dy, string, color);
}

static void draw_rectangle(GLint x, GLint y, GLint w, GLint h, GLfloat *c) {
	glBegin(GL_QUADS);
		glColor4fv(c);
		glVertex2i(x  , y  );
		glVertex2i(x+w, y  );
		glVertex2i(x+w, y+h);
		glVertex2i(x  , y+h);
	glEnd();
}

static void draw_shaded_rectangle(GLint x, GLint y, GLint w, GLint h, GLfloat *color) {
	glBegin(GL_QUADS);
		glColor4fv(color);
		glVertex2i(x,  y);

		glColor3f(0.8, 0.8, 0.8);
		glVertex2i(x+w,y);
		
		glColor4fv(color);
		glVertex2i(x+w,y+h);
		
		glColor3f(0.3, 0.3, 0.3);
		glVertex2i(x,  y+h);
	glEnd();
}

static void draw_button_face(GLint x, GLint y, GLint w, GLint h,
			     const char *string, GLenum halign, GLenum valign,
			     GLfloat back_color[], GLfloat text_color[]) {

	draw_shaded_rectangle(x,y,w,h, back_color);
	draw_aligned_string(x,y,w,h, string, halign, valign, text_color);
}
